@@ -10,10 +10,13 @@ try:
         teardown_session,
         scenario_picker,
         events,
+        session_factory,
     )
     from molotov.util import request, json_request
     from molotov.util import set_var, get_var
+    from molotov.session import get_context
+    from molotov import _grpc  # register GRPC factory
 except ImportError:
     pass  # first import
 
-__version__ = "2.1"
+__version__ = "2.7"
