@@ -1,5 +1,7 @@
 import io
+
 import aiohttp
+from aiohttp.streams import DataQueue
 
 from molotov.api import get_fixture
 
@@ -9,7 +11,7 @@ _FILE = "**** File content ****"
 _COMPRESSED = ("gzip", "compress", "deflate", "identity", "br")
 
 
-class BaseListener(object):
+class BaseListener:
     async def __call__(self, event, **options):
         attr = getattr(self, "on_" + event, None)
         if attr is not None:
@@ -27,6 +29,7 @@ class Writer:
 class StdoutListener(BaseListener):
     def __init__(self, **options):
         self.verbose = options.get("verbose", 0)
+        self.loop = options.pop("loop", None)
         self.console = options["console"]
 
     async def _body2str(self, body):
@@ -60,10 +63,9 @@ class StdoutListener(BaseListener):
     async def on_sending_request(self, session, request):
         if self.verbose < 2:
             return
-        raw = ">" * 45
-        raw += "\n" + request.method + " " + str(request.url)
+        raw = "\n" + request.method + " " + str(request.url)
         if len(request.headers) > 0:
-            headers = "\n".join("%s: %s" % (k, v) for k, v in request.headers.items())
+            headers = "\n".join("{}: {}".format(k, v) for k, v in request.headers.items())
             raw += "\n" + headers
         if request.headers.get("Content-Encoding") in _COMPRESSED:
             raw += "\n\n" + _BINARY + "\n"
@@ -71,15 +73,14 @@ class StdoutListener(BaseListener):
             str_body = await self._body2str(request.body)
             raw += "\n\n" + str_body + "\n"
 
-        self.console.print(raw)
+        self.console.print_error(raw)
 
     async def on_response_received(self, session, response, request):
         if self.verbose < 2:
             return
-        raw = "\n" + "=" * 45 + "\n"
-        raw += "HTTP/1.1 %d %s\n" % (response.status, response.reason)
+        raw = "HTTP/1.1 %d %s\n" % (response.status, response.reason)
         items = response.headers.items()
-        headers = "\n".join("{}: {}".format(k, v) for k, v in items)
+        headers = "\n".join(f"{k}: {v}" for k, v in items)
         raw += headers
         if response.headers.get("Content-Encoding") in _COMPRESSED:
             raw += "\n\n" + _BINARY
@@ -87,7 +88,9 @@ class StdoutListener(BaseListener):
             content = await response.content.read()
             if len(content) > 0:
                 # put back the data in the content
-                response.content.unread_data(content)
+                response.content = DataQueue(loop=self.loop)
+                response.content.feed_data(content)
+                response.content.feed_eof()
                 try:
                     raw += "\n\n" + content.decode()
                 except UnicodeDecodeError:
@@ -95,11 +98,12 @@ class StdoutListener(BaseListener):
             else:
                 raw += "\n\n"
 
-        raw += "\n" + "<" * 45 + "\n"
-        self.console.print(raw)
+        self.console.print_error(raw)
+        self.console.print_error("")
+        self.console.print_error("")
 
 
-class CustomListener(object):
+class CustomListener:
     def __init__(self, fixture):
         self.fixture = fixture
 
@@ -107,7 +111,7 @@ class CustomListener(object):
         await self.fixture(event, **options)
 
 
-class EventSender(object):
+class EventSender:
     def __init__(self, console, listeners=None):
         self.console = console
         if listeners is None:
@@ -129,9 +133,9 @@ class EventSender(object):
     def stopped(self):
         return self._stopped
 
-    async def send_event(self, event, **options):
+    async def send_event(self, event, *args, **options):
         for listener in self._listeners:
             try:
-                await listener(event, **options)
+                await listener(event, *args, **options)
             except Exception as e:
                 self.console.print_error(e)
