@@ -12,9 +12,8 @@ This Molotov script has:
 import molotov
 
 
-class SomeObject(object):
-    """Does something smart in real life with the async loop.
-    """
+class SomeObject:
+    """Does something smart in real life with the async loop."""
 
     def __init__(self, loop):
         self.loop = loop
@@ -37,7 +36,7 @@ async def init_worker(worker_num, args):
 
 @molotov.setup_session()
 async def init_session(worker_num, session):
-    session.ob = SomeObject(loop=session.loop)
+    molotov.get_context(session).attach("ob", SomeObject(loop=session.loop))
 
 
 @molotov.scenario(100)
@@ -51,7 +50,7 @@ async def scenario_one(session):
 
 @molotov.teardown_session()
 async def end_session(worker_num, session):
-    session.ob.cleanup()
+    molotov.get_context(session).ob.cleanup()
 
 
 @molotov.teardown()
