@@ -1,5 +1,10 @@
 from molotov.listeners import BaseListener, EventSender
-from molotov.tests.support import TestLoop, async_test, serialize
+from molotov.tests.support import TestLoop, async_test, patch_errors
+
+
+class MyBuggyListener(BaseListener):
+    def on_my_event(self, **options):
+        raise Exception("Bam")
 
 
 class TestListeners(TestLoop):
@@ -18,20 +23,15 @@ class TestListeners(TestLoop):
         eventer = EventSender(console)
         eventer.add_listener(listener)
         await eventer.send_event("my_event", value=42)
-        await serialize(console)
-
         self.assertTrue(listener.fired)
         self.assertEqual(listener.value, 42)
 
+    @patch_errors
     @async_test
-    async def test_buggy_listener(self, loop, console, results):
-        class MyListener(BaseListener):
-            def on_my_event(self, **options):
-                raise Exception("Bam")
-
-        listener = MyListener()
+    async def test_buggy_listener(self, console_print, loop, console, results):
+        listener = MyBuggyListener()
         eventer = EventSender(console)
         eventer.add_listener(listener)
         await eventer.send_event("my_event")
-        resp = await serialize(console)
-        self.assertTrue("Bam" in resp)
+
+        self.assertTrue("Bam" in console_print())
