@@ -1,14 +1,14 @@
-import tempfile
-import shutil
 import os
+import shutil
+import tempfile
 
-from molotov import quickstart, __version__, run
-from molotov.tests.support import set_args, TestLoop, dedicatedloop
+from molotov import __version__, quickstart, run
+from molotov.tests.support import TestLoop, dedicatedloop, set_args
 
 
 class TestQuickStart(TestLoop):
     def setUp(self):
-        super(TestQuickStart, self).setUp()
+        super().setUp()
         self._curdir = os.getcwd()
         self.tempdir = tempfile.mkdtemp()
         self.location = os.path.join(self.tempdir, "new")
@@ -17,7 +17,7 @@ class TestQuickStart(TestLoop):
     def tearDown(self):
         os.chdir(self._curdir)
         shutil.rmtree(self.tempdir)
-        super(TestQuickStart, self).tearDown()
+        super().tearDown()
 
     def _input(self, text):
         if self._answers == []:
@@ -66,7 +66,7 @@ class TestQuickStart(TestLoop):
         self.assertEqual(result, ["Makefile", "loadtest.py", "molotov.json"])
 
         os.chdir(self.location)
-        with set_args("molotov", "-cxv", "--max-runs", "1"):
+        with set_args("molotov", "-xv", "--max-runs", "1"):
             try:
                 run.main()
             except SystemExit:
