@@ -1,10 +1,12 @@
+import os
 import unittest
-import multiprocessing
-from molotov.sharedcounter import SharedCounters, SharedCounter
 
+import multiprocess
+
+from molotov.shared.counter import Counter, Counters
 
 # pre-forked variable
-_DATA = SharedCounters("test")
+_DATA = Counters("test")
 
 
 def run_worker(value):
@@ -13,10 +15,10 @@ def run_worker(value):
     _DATA["test"] += value
 
 
-class TestSharedCounters(unittest.TestCase):
+class TestCounters(unittest.TestCase):
     def test_operators(self):
-        c1 = SharedCounter("ok")
-        c2 = SharedCounter("ok")
+        c1 = Counter("ok")
+        c2 = Counter("ok")
         c1.value = 4
         c2.value = 5
         self.assertTrue(c1 <= c2)
@@ -26,24 +28,24 @@ class TestSharedCounters(unittest.TestCase):
         self.assertTrue(c1 == 4)
         self.assertTrue(c1 != 5)
         c2.value = 4
-        c2 += SharedCounter("ok")
+        c2 += Counter("ok")
         self.assertTrue(c1 == c2)
         repr(c1)
         str(c1)
 
         def _t():
-            c = SharedCounter("ok")
+            c = Counter("ok")
             c += 6.2
 
         self.assertRaises(NotImplementedError, _t)
 
         def _c():
-            SharedCounter("ok") != 6.3
+            Counter("ok") != 6.3  # noqa
 
         self.assertRaises(TypeError, _c)
 
     def test_interface(self):
-        data = SharedCounters("one", "two")
+        data = Counters("one", "two")
         self.assertTrue("one" in data)
         self.assertEqual(len(data.keys()), 2)
 
@@ -63,7 +65,7 @@ class TestSharedCounters(unittest.TestCase):
 
     def test_mapping(self):
         # making sure it works like a defaultdict(int)
-        data = SharedCounters("one", "two")
+        data = Counters("one", "two")
         self.assertTrue(data["one"].value == 0)
         data["one"] += 10
         data["one"] -= 1
@@ -71,9 +73,10 @@ class TestSharedCounters(unittest.TestCase):
         self.assertTrue(data["one"].value, 9)
         self.assertTrue(data["two"].value, 4)
 
+    @unittest.skipIf(os.name == "nt", "win32")
     def test_multiprocess(self):
         # now let's try with several processes
-        pool = multiprocessing.Pool(10)
+        pool = multiprocess.Pool(10)
         try:
             inputs = [1] * 3000
             pool.map(run_worker, inputs)
