@@ -1,9 +1,10 @@
+import json
+import os
+import unittest
 from io import StringIO
 from tempfile import mkstemp
-import json
-import unittest
-import os
-from molotov.util import resolve, expand_options, OptionError, set_var, get_var, _VARS
+
+from molotov.util import _VARS, OptionError, expand_options, get_var, set_var
 
 _HERE = os.path.dirname(__file__)
 config = os.path.join(_HERE, "..", "..", "molotov.json")
@@ -15,27 +16,9 @@ class Args:
 
 class TestUtil(unittest.TestCase):
     def setUp(self):
-        super(TestUtil, self).setUp()
+        super().setUp()
         _VARS.clear()
 
-    def test_resolve(self):
-
-        urls = [
-            ("http://localhost:80/blah", "http://127.0.0.1:80/blah"),
-            ("https://localhost", "https://localhost"),
-            ("http://cantfind", "http://cantfind"),
-            ("https://google.com", "https://google.com"),
-            (
-                "http://user:pass@localhost/blah?yeah=1#ok",
-                "http://user:pass@127.0.0.1/blah?yeah=1#ok",
-            ),
-            ("http://tarek@localhost/blah", "http://tarek@127.0.0.1/blah"),
-        ]
-
-        for url, wanted in urls:
-            changed, original, resolved = resolve(url)
-            self.assertEqual(changed, wanted, "%s vs %s" % (original, resolved))
-
     def test_config(self):
         args = Args()
         expand_options(config, "test", args)
